(*
 * Js_of_ocaml による Semi-discretization method の学習
 *)

module Html = Dom_html
let jstr = Js.string
let document = Html.window##document

let create_canvas () =
  let c = Html.createCanvas document in
  c##width <- 320;
  c##height <- 320;
  c
let canvas = create_canvas ()

(* 浮動小数入力テキストボックス *)
let float_input name value =
  let res = document##createDocumentFragment() in
  Dom.appendChild res (document##createTextNode(jstr name));
  let input = Html.createInput ~_type:(jstr "text") document in
  input##value <- jstr (string_of_float !value);
  input##onchange <- Html.handler
    (fun _ ->
       begin try
         value := float_of_string (Js.to_string (input##value))
       with Invalid_argument _ ->
         ()
       end;
       input##value <- jstr (string_of_float !value);
       Js._false);
  Dom.appendChild res input;
  res

(* ボタン *)
let button name callback =
  let res = document##createDocumentFragment() in
  let input = Html.createInput ~_type:(jstr "submit") document in
  input##value <- jstr name;
  input##onclick <- Html.handler callback;
  Dom.appendChild res input;
  res


let zeta_in = ref 0.05 (* 減衰 *)
let a0_in = ref 1.0    (* ばね定数 *)
let b0_in = ref 0.2    (* 切削抵抗 *)


let tpi = 8. *. atan 1. (* 2π *)

(* 
 * plotting points
 * 計算結果の各点をプロット
 *)
let plot_points ctx lcor arr =
  ctx##strokeStyle <- lcor;
  ctx##beginPath ();
  ctx##moveTo (20. , 200.);
  Array.iteri (fun n v -> ctx##lineTo (20. +. (float n) *. 0.2, 200. -. 100. *. v)) arr;
  ctx##stroke ()

(*
 * drawing x-axis
 * x軸描画
 *)
let draw_x_axis ctx =
  ctx##strokeStyle <- Js.string "black";
  ctx##beginPath ();
  ctx##moveTo (10., 200.);
  ctx##lineTo (300., 200.);
  ctx##stroke ()

(*
 * drawing lines of graph
 * グラフの各点の情報から線を描画
 *)
let draw_graphs () =
  let ctx = canvas##getContext (Html._2d_) in
  ctx##clearRect (0., 0., float canvas##width, float canvas##height);
  let seq_0 = Semidiscrete.init_seq 0.05 1.0 0.0 in
  let seq_1 = Semidiscrete.init_seq !zeta_in !a0_in !b0_in in
  draw_x_axis ctx;
  Semidiscrete.seq_to_array seq_0 1000 |> plot_points ctx (Js.string "blue"); 
  Semidiscrete.seq_to_array seq_1 1000 |> plot_points ctx (Js.string "green");
  ()

(*
 * HTML ロード時の処理
 *)
let onload _ =
  let main =
    Js.Opt.get (document##getElementById(jstr "main"))
      (fun () -> assert false)
  in
  Dom.appendChild main (float_input "zeta = " zeta_in);
  Dom.appendChild main (Html.createBr document);
  Dom.appendChild main (float_input "a0 = " a0_in);
  Dom.appendChild main (Html.createBr document);
  Dom.appendChild main (float_input "b0 = " b0_in);
  Dom.appendChild main (Html.createBr document);
  Dom.appendChild main
    (button "display" (fun _ -> draw_graphs(); Js._false));
  canvas##width <- 320;
  canvas##height <- 320;
  Dom.appendChild Html.window##document##body canvas;
  draw_graphs ();
  Js._false

(*
 * entry point
 * 最初に実行される処理
 *)
let () = Html.window##onload <- Html.handler onload
